@@ -2,7 +2,7 @@
 
 -- |
 -- Module      : Data.Primitive.Addr
--- Copyright   : (c) Roman Leshchinskiy 2009-2010
+-- Copyright   : (c) Roman Leshchinskiy 2009-2012
 -- License     : BSD-style
 --
 -- Maintainer  : Roman Leshchinskiy <rl@cse.unsw.edu.au>
@@ -12,11 +12,17 @@
 --
 
 module Data.Primitive.Addr (
+  -- * Types
   Addr(..),
 
+  -- * Address arithmetic
   nullAddr, plusAddr, minusAddr, remAddr,
+
+  -- * Element access
   indexOffAddr, readOffAddr, writeOffAddr,
-  copyAddr, moveAddr, memcpyAddr
+
+  -- * Block operations
+  copyAddr, moveAddr, setAddr
 ) where
 
 import Control.Monad.Primitive
@@ -88,8 +94,9 @@ moveAddr :: PrimMonad m => Addr         -- ^ destination address
 moveAddr (Addr dst#) (Addr src#) n
   = unsafePrimToPrim $ moveBytes (Ptr dst#) (Ptr src#) n
 
-memcpyAddr :: PrimMonad m => Addr -> Addr -> Int -> m ()
-{-# INLINE memcpyAddr #-}
-{-# DEPRECATED memcpyAddr "Use copyAddr instead" #-}
-memcpyAddr = copyAddr
+-- | Fill a memory block of with the given value. The length is in
+-- elements of type @a@ rather than in bytes.
+setAddr :: (Prim a, PrimMonad m) => Addr -> Int -> a -> m ()
+{-# INLINE setAddr #-}
+setAddr (Addr addr#) (I# n#) x = primitive_ (setOffAddr# addr# 0# n# x)
 
