@@ -3,7 +3,7 @@
 
 -- |
 -- Module      : Data.Primitive.ByteArray
--- Copyright   : (c) Roman Leshchinskiy 2009-2010
+-- Copyright   : (c) Roman Leshchinskiy 2009-2012
 -- License     : BSD-style
 --
 -- Maintainer  : Roman Leshchinskiy <rl@cse.unsw.edu.au>
@@ -13,17 +13,25 @@
 --
 
 module Data.Primitive.ByteArray (
+  -- * Types
   ByteArray(..), MutableByteArray(..), ByteArray#, MutableByteArray#,
 
+  -- * Allocation
   newByteArray, newPinnedByteArray, newAlignedPinnedByteArray,
-  readByteArray, writeByteArray, indexByteArray,
-  unsafeFreezeByteArray, unsafeThawByteArray,
-  sizeofByteArray, sizeofMutableByteArray, sameMutableByteArray,
-  byteArrayContents, mutableByteArrayContents,
-  copyByteArray, copyMutableByteArray, moveByteArray, fillByteArray,
 
-  -- * Deprecated operations
-  memcpyByteArray, memcpyByteArray', memmoveByteArray, memsetByteArray
+  -- * Element access
+  readByteArray, writeByteArray, indexByteArray,
+
+  -- * Freezing and thawing
+  unsafeFreezeByteArray, unsafeThawByteArray,
+
+  -- * Block operations
+  copyByteArray, copyMutableByteArray, moveByteArray,
+  setByteArray, fillByteArray,
+
+  -- * Information
+  sizeofByteArray, sizeofMutableByteArray, sameMutableByteArray,
+  byteArrayContents, mutableByteArrayContents
 ) where
 
 import Control.Monad.Primitive
@@ -33,6 +41,9 @@ import Foreign.C.Types
 import Data.Word ( Word8 )
 import GHC.Base ( Int(..) )
 import GHC.Prim
+#if __GLASGOW_HASKELL__ >= 706
+    hiding (setByteArray#)
+#endif
 
 import Data.Typeable ( Typeable )
 import Data.Data ( Data(..) )
@@ -203,71 +214,46 @@ moveByteArray (MutableByteArray dst#) doff
   $ memmove_mba dst# (fromIntegral doff) src# (fromIntegral soff)
                      (fromIntegral sz)
 
--- | Fill a slice of a mutable byte array with a value.
+-- | Fill a slice of a mutable byte array with a value. The offset and length
+-- are given in elements of type @a@ rather than in bytes.
+setByteArray
+  :: (Prim a, PrimMonad m) => MutableByteArray (PrimState m) -- ^ array to fill
+                           -> Int                 -- ^ offset into array
+                           -> Int                 -- ^ number of values to fill
+                           -> a                   -- ^ value to fill with
+                           -> m ()
+{-# INLINE setByteArray #-}
+setByteArray (MutableByteArray dst#) (I# doff#) (I# sz#) x
+  = primitive_ (setByteArray# dst# doff# sz# x)
+
+-- | Fill a slice of a mutable byte array with a byte.
 fillByteArray
   :: PrimMonad m => MutableByteArray (PrimState m)
                                         -- ^ array to fill
                  -> Int                 -- ^ offset into array
                  -> Int                 -- ^ number of bytes to fill
-                 -> Word8               -- ^ value to fill with
+                 -> Word8               -- ^ byte to fill with
                  -> m ()
 {-# INLINE fillByteArray #-}
-fillByteArray (MutableByteArray dst#) doff sz c
-  = unsafePrimToPrim
-  $ memset_mba dst# (fromIntegral doff) (fromIntegral c) (fromIntegral sz)
+fillByteArray = setByteArray
 
 
 
-memcpyByteArray
-  :: PrimMonad m => MutableByteArray (PrimState m) -> Int
-                 -> MutableByteArray (PrimState m) -> Int
-                 -> Int -> m ()
-{-# DEPRECATED memcpyByteArray "Use copyMutableByteArray instead" #-}
-{-# INLINE memcpyByteArray #-}
-memcpyByteArray = copyMutableByteArray
-
-memcpyByteArray'
-  :: PrimMonad m => MutableByteArray (PrimState m) -> Int
-                 -> ByteArray -> Int
-                 -> Int -> m ()
-{-# DEPRECATED memcpyByteArray' "Use copyByteArray instead" #-}
-{-# INLINE memcpyByteArray' #-}
-memcpyByteArray' = copyByteArray
-
-memmoveByteArray
-  :: PrimMonad m => MutableByteArray (PrimState m) -> Int
-                 -> MutableByteArray (PrimState m) -> Int
-                 -> Int -> m ()
-{-# DEPRECATED memmoveByteArray "Use moveByteArray instead" #-}
-{-# INLINE memmoveByteArray #-}
-memmoveByteArray = moveByteArray
-
-memsetByteArray
-  :: PrimMonad m => MutableByteArray (PrimState m) -> Int -> Word8
-                 -> Int -> m ()
-{-# DEPRECATED memsetByteArray "Use fillByteArray instead (WARNING: order of arguments is different)" #-}
-{-# INLINE memsetByteArray #-}
-memsetByteArray dst off x sz = fillByteArray dst off sz x
-
-
-foreign import ccall unsafe "primitive-memops.h memcpy_off"
+foreign import ccall unsafe "primitive-memops.h hsprimitive_memcpy"
   memcpy_mba :: MutableByteArray# s -> CInt
              -> MutableByteArray# s -> CInt
              -> CSize -> IO ()
 
-foreign import ccall unsafe "primitive-memops.h memcpy_off"
+foreign import ccall unsafe "primitive-memops.h hsprimitive_memcpy"
   memcpy_ba :: MutableByteArray# s -> CInt
             -> ByteArray# -> CInt
             -> CSize -> IO ()
 
-foreign import ccall unsafe "primitive-memops.h memmove_off"
+foreign import ccall unsafe "primitive-memops.h hsprimitive_memmove"
   memmove_mba :: MutableByteArray# s -> CInt
               -> MutableByteArray# s -> CInt
               -> CSize -> IO ()
 
-foreign import ccall unsafe "primitive-memops.h memset_off"
-  memset_mba :: MutableByteArray# s -> CInt -> CInt -> CSize -> IO ()
-
 instance Data ByteArray where
   toConstr _ = error "toConstr"
   gunfold _ _ = error "gunfold"
