@@ -1,6 +1,7 @@
+{-# LANGUAGE MagicHash #-}
 -- |
 -- Module      : Data.Primitive.MachDeps
--- Copyright   : (c) Roman Leshchinskiy 2009
+-- Copyright   : (c) Roman Leshchinskiy 2009-2012
 -- License     : BSD-style
 --
 -- Maintainer  : Roman Leshchinskiy <rl@cse.unsw.edu.au>
@@ -13,6 +14,8 @@ module Data.Primitive.MachDeps where
 
 #include "MachDeps.h"
 
+import GHC.Prim
+
 sIZEOF_CHAR,
  aLIGNMENT_CHAR,
 
@@ -110,3 +113,11 @@ aLIGNMENT_INT64 = ALIGNMENT_INT64
 sIZEOF_WORD64 = SIZEOF_WORD64
 aLIGNMENT_WORD64 = ALIGNMENT_WORD64
 
+#if WORD_SIZE_IN_BITS == 32
+type Word64_# = Word64#
+type Int64_# = Int64#
+#else
+type Word64_# = Word#
+type Int64_# = Int#
+#endif
+
